@@ -22,12 +22,22 @@ level, which is held in the debugLevel field.  All other methods take
 a debug level, which is compared to the current debug level to
 determine if the content should be output."""
 
-__id__        = "$Id: debug.py 1036 2006-06-10 00:54:48Z wwalker $"
-__version__   = "$Revision: 1036 $"
-__date__      = "$Date: 2006-06-09 20:54:48 -0400 (Fri, 09 Jun 2006) $"
+__id__        = "$Id: debug.py,v 1.30 2006/10/13 23:39:04 wwalker Exp $"
+__version__   = "$Revision: 1.30 $"
+__date__      = "$Date: 2006/10/13 23:39:04 $"
 __copyright__ = "Copyright (c) 2005-2006 Sun Microsystems Inc."
 __license__   = "LGPL"
 
+import logging
+logging.basicConfig()
+
+log = logging.getLogger()
+
+# Set the level to INFO if you want the logging of speech and braille
+# output.
+#
+#log.setLevel(logging.INFO)
+
 import traceback
 
 # Used to turn off all debugging.
