@@ -19,12 +19,15 @@
 
 """Provides a SpeechServer factory for gnome-speech drivers."""
 
-__id__        = "$Id: gnomespeechfactory.py,v 1.53 2006/09/08 15:48:58 richb Exp $"
-__version__   = "$Revision: 1.53 $"
-__date__      = "$Date: 2006/09/08 15:48:58 $"
+__id__        = "$Id: gnomespeechfactory.py,v 1.57 2006/10/27 16:25:43 mpeders Exp $"
+__version__   = "$Revision: 1.57 $"
+__date__      = "$Date: 2006/10/27 16:25:43 $"
 __copyright__ = "Copyright (c) 2005-2006 Sun Microsystems Inc."
 __license__   = "LGPL"
 
+import logging
+log = logging.getLogger("speech")
+
 import os
 
 import gobject
@@ -36,14 +39,16 @@ import time
 import bonobo
 
 import atspi
+import chnames
 import debug
+import orca_state
 import punctuation_settings
 import settings
 import speech
 import speechserver
+import util
 
 from acss import ACSS
-from chnames import chnames
 
 from orca_i18n import _           # for gettext support
 
@@ -454,6 +459,8 @@ class SpeechServer(speechserver.SpeechServer):
                                       "SPEECH OUTPUT: '" \
                                       + self.__sayAll.currentContext.utterance\
                                       + "'")
+                        log.info("'%s'" \
+                                 % self.__sayAll.currentContext.utterance)
                         self.__sayAll.idForCurrentContext = self.__speak(
                             self.__sayAll.currentContext.utterance,
                             acss)
@@ -584,7 +591,7 @@ class SpeechServer(speechserver.SpeechServer):
         """
         i = 0
         for text in list:
-            if len(text) != 0:
+            if len(text):
                 self.speak(text, acss, interrupt and (i == 0))
             i += 1
 
@@ -605,16 +612,31 @@ class SpeechServer(speechserver.SpeechServer):
         Returns a text string with the punctuation symbols adjusted accordingly.
         """
 
-        ## Replace ellipses (both manual and unicode) with "dot dot dot"
+        ## Replace ellipses (both manual and UTF-8) with "dot dot dot"
         ##
         oldText = oldText.replace("...", _(" dot dot dot"), 1)
         oldText = oldText.replace("\342\200\246",  _(" dot dot dot"), 1)
+        oldText = oldText.decode("UTF-8")
+
+        ## Don't speak newlines unless the user is moving through text
+        ## using a right or left arrow key.
+        ##
+        removeNewLines = True
+        if orca_state.lastInputEvent and \
+               orca_state.lastInputEvent.__dict__.has_key("event_string"):
+            lastKey = orca_state.lastInputEvent.event_string
+            if lastKey == "Left" or lastKey == "Right":
+                removeNewLines = False
+
+        if removeNewLines:
+            oldText = oldText.replace("\n", _(""), 1)
 
         style = settings.verbalizePunctuationStyle
         newText = ''
         for i in range(0, len(oldText)):
             try:
-                level, action = punctuation_settings.punctuation[oldText[i]]
+                level, action = \
+                    punctuation_settings.getPunctuationInfo(oldText[i])
 
                 # Special case for punctuation in text like filenames or URL's:
                 #
@@ -632,19 +654,19 @@ class SpeechServer(speechserver.SpeechServer):
                 if (len(oldText) == 1) or isSpecial or (style <= level):
                     if isPrev:
                         newText += " "
-                    newText += chnames[oldText[i]]
+                    newText += chnames.getCharacterName(oldText[i])
                     if (action == punctuation_settings.PUNCTUATION_INSERT) \
                         and not isNext:
-                        newText += oldText[i]
+                        newText += oldText[i].encode("UTF-8")
                     if isNext:
                         newText += " "
                 else:
-                    newText += oldText[i]
+                    newText += oldText[i].encode("UTF-8")
             except:
-                if (len(oldText) == 1) and chnames.has_key(oldText[i]):
-                    newText += chnames[oldText[i].lower()]
+                if (len(oldText) == 1):
+                    newText += chnames.getCharacterName(oldText[i])
                 else:
-                    newText += oldText[i]
+                    newText += oldText[i].encode("UTF-8")
 
         return newText
 
@@ -684,6 +706,7 @@ class SpeechServer(speechserver.SpeechServer):
             return -1
 
         text = self.__addVerbalizedPunctuation(text)
+        text = util.adjustForPronunciation(text)
 
         try:
             # [[[TODO: WDW - back this stop out for now.  The problem is
@@ -741,6 +764,7 @@ class SpeechServer(speechserver.SpeechServer):
             [context, acss] = utteranceIterator.next()
             debug.println(debug.LEVEL_INFO,
                           "SPEECH OUTPUT: '" + context.utterance + "'")
+            log.info("'%s'" % context.utterance)
             self.__sayAll = _SayAll(utteranceIterator,
                                     context,
                                     self.__speak(context.utterance, acss),
