@@ -1,6 +1,6 @@
 # Orca
 #
-# Copyright 2005 Sun Microsystems Inc.
+# Copyright 2005-2006 Sun Microsystems Inc.
 #
 # This library is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Library General Public
@@ -23,9 +23,9 @@ MouseButtonEvent, MouseMotionEvent, and SpeechEvent), and also provides a
 InputEventHandler class.  It is intended that instances of InputEventHandler
 will be used which should be used to handle all input events."""
 
-__id__        = "$Id: input_event.py,v 1.13 2006/07/28 17:33:16 wwalker Exp $"
-__version__   = "$Revision: 1.13 $"
-__date__      = "$Date: 2006/07/28 17:33:16 $"
+__id__        = "$Id: input_event.py,v 1.15 2006/10/05 16:31:03 wwalker Exp $"
+__version__   = "$Revision: 1.15 $"
+__date__      = "$Date: 2006/10/05 16:31:03 $"
 __copyright__ = "Copyright (c) 2005-2006 Sun Microsystems Inc."
 __license__   = "LGPL"
 
@@ -67,7 +67,7 @@ class KeyboardEvent(InputEvent):
         # be turned into the string "A").  All these checks here are
         # to just do some sanity checking before doing the
         # conversion. [[[WDW - this is making assumptions about
-        # mapping ASCII control characters to to UTF-8.]]]
+        # mapping ASCII control characters to UTF-8.]]]
         #
         event_string = event.event_string
         if (event.modifiers & (1 << atspi.Accessibility.MODIFIER_CONTROL)) \
@@ -125,7 +125,7 @@ class SpeechEvent(InputEvent):
 
 class InputEventHandler:
 
-    def __init__(self, function, description):
+    def __init__(self, function, description, learnModeEnabled=True):
         """Creates a new InputEventHandler instance.  All bindings
         (e.g., key bindings and braille bindings) will be handled
         by an instance of an InputEventHandler.
@@ -137,10 +137,14 @@ class InputEventHandler:
                     False
         - description: a localized string describing what this InputEvent
                        does
+        - learnModeEnabled: if True, the description will be spoken and
+                            brailled if learn mode is enabled.  If False,
+                            the function will be called no matter what.
         """
 
         self._function = function
         self._description = description
+        self._learnModeEnabled = learnModeEnabled
 
     def processInputEvent(self, script, inputEvent):
         """Processes an input event.  If settings.learnModeEnabled is True,
@@ -160,7 +164,7 @@ class InputEventHandler:
 
         consumed = False
 
-        if settings.learnModeEnabled:
+        if settings.learnModeEnabled and self._learnModeEnabled:
             if self._description:
                 # These imports are here to eliminate circular imports.
                 #
