@@ -20,9 +20,9 @@
 """Manages the magnifier for orca.  [[[TODO: WDW - this is very very
 early in development.  One might even say it is pre-prototype.]]]"""
 
-__id__        = "$Id: mag.py 1339 2006-08-18 17:39:17Z wwalker $"
-__version__   = "$Revision: 1339 $"
-__date__      = "$Date: 2006-08-18 13:39:17 -0400 (Fri, 18 Aug 2006) $"
+__id__        = "$Id: mag.py,v 1.31 2006/09/19 21:30:55 richb Exp $"
+__version__   = "$Revision: 1.31 $"
+__date__      = "$Date: 2006/09/19 21:30:55 $"
 __copyright__ = "Copyright (c) 2005-2006 Sun Microsystems Inc."
 __license__   = "LGPL"
 
@@ -285,12 +285,12 @@ def applySettings():
     _magnifier.clearAllZoomRegions()
 
     try:
-        _magnifier.TargetDisplay = settings.magnifierTargetDisplay
+        _magnifier.TargetDisplay = settings.magTargetDisplay
     except:
         pass
 
     try:
-        _magnifier.SourceDisplay = settings.magnifierSourceDisplay
+        _magnifier.SourceDisplay = settings.magSourceDisplay
     except:
         pass
 
