@@ -19,9 +19,9 @@
 
 """The main module for the Orca screen reader."""
 
-__id__        = "$Id: orca.py,v 1.160 2006/10/02 19:28:56 richb Exp $"
-__version__   = "$Revision: 1.160 $"
-__date__      = "$Date: 2006/10/02 19:28:56 $"
+__id__        = "$Id: orca.py,v 1.164 2006/11/02 16:07:42 richb Exp $"
+__version__   = "$Revision: 1.164 $"
+__date__      = "$Date: 2006/11/02 16:07:42 $"
 __copyright__ = "Copyright (c) 2005-2006 Sun Microsystems Inc."
 __license__   = "LGPL"
 
@@ -616,8 +616,8 @@ def _keyEcho(event):
 
         # Check to see if there are localized words to be spoken for
         # this key event.
-        if event_string in keynames.keynames:
-            event_string = keynames.keynames[event_string]
+        #
+        event_string = keynames.getKeyName(event_string)
 
         if lockState:
             event_string += lockState
@@ -706,14 +706,18 @@ def _processKeyboardEvent(event):
         if (not consumed) and settings.learnModeEnabled:
             if keyboardEvent.type \
                 == atspi.Accessibility.KEY_PRESSED_EVENT:
-                # Check to see if there are localized words to be
-                # spoken for this key event.
-                #
-                braille.displayMessage(keyboardEvent.event_string)
-                event_string = keyboardEvent.event_string
-                if event_string in keynames.keynames:
-                    event_string = keynames.keynames[event_string]
-                speech.speak(event_string)
+                clickCount = util.getClickCount(orca_state.lastInputEvent,
+                                            keyboardEvent)
+                if clickCount == 2:
+                    util.phoneticSpellCurrentItem(keyboardEvent.event_string)
+                else:
+                    # Check to see if there are localized words to be
+                    # spoken for this key event.
+                    #
+                    braille.displayMessage(keyboardEvent.event_string)
+                    event_string = keyboardEvent.event_string
+                    event_string = keynames.getKeyName(event_string)
+                    speech.speak(event_string)
             consumed = True
     except:
         debug.printException(debug.LEVEL_SEVERE)
@@ -1190,6 +1194,7 @@ def main():
     try:
         # ? is for help
         # h is for help
+        # u is for alternate user preferences location
         # s is for setup
         # n is for no setup
         # t is for text setup
@@ -1197,14 +1202,22 @@ def main():
         #
         opts, args = getopt.getopt(
             sys.argv[1:],
-            "?stnv",
+            "?u:stnv",
             ["help",
+             "user-prefs-dir",
              "setup",
              "gui-setup",
              "text-setup",
              "no-setup",
              "version"])
         for opt, val in opts:
+            if opt in ("-u", "--user-prefs-dir"):
+                userPrefsDir = val.strip();
+                try:
+                    os.chdir(userPrefsDir)
+                    settings.userPrefsDir = userPrefsDir
+                except:
+                    debug.printException(debug.LEVEL_FINEST)
             if opt in ("-s", "--gui-setup", "--setup"):
                 setupRequested = True
                 showGUI = desktopRunning
@@ -1241,7 +1254,7 @@ def main():
         print "environment variable has been set."
         return 1
 
-    userprefs = os.path.join(os.environ["HOME"], ".orca")
+    userprefs = settings.userPrefsDir
     sys.path.insert(0, userprefs)
     sys.path.insert(0, '') # current directory
 
