@@ -33,15 +33,17 @@ created in their own module.  The module defining the Script subclass
 is also required to have a 'getScript(app)' method that returns an
 instance of the Script subclass.  See default.py for an example."""
 
-__id__        = "$Id: script.py,v 1.37 2006/10/12 00:33:47 wwalker Exp $"
-__version__   = "$Revision: 1.37 $"
-__date__      = "$Date: 2006/10/12 00:33:47 $"
+__id__        = "$Id: script.py,v 1.40 2006/10/22 20:51:44 wwalker Exp $"
+__version__   = "$Revision: 1.40 $"
+__date__      = "$Date: 2006/10/22 20:51:44 $"
 __copyright__ = "Copyright (c) 2005-2006 Sun Microsystems Inc."
 __license__   = "LGPL"
 
 import braillegenerator
 import debug
+import flat_review
 import keybindings
+import orca_state
 import settings
 import speechgenerator
 
@@ -69,6 +71,10 @@ class Script:
 
         self.listeners = self.getListeners()
 
+        # By default, handle events for non-active applications.
+        #
+        self.presentIfInactive = True
+
         self.inputEventHandlers = {}
         self.setupInputEventHandlers()
         self.keyBindings = self.getKeyBindings()
@@ -78,6 +84,8 @@ class Script:
         self.speechGenerator = self.getSpeechGenerator()
         self.voices = settings.voices
 
+        self.flatReviewContextClass = flat_review.Context
+
         debug.println(debug.LEVEL_FINE, "NEW SCRIPT: %s" % self.name)
 
     def getListeners(self):
@@ -182,6 +190,14 @@ class Script:
         - event: the Event
         """
 
+        # Check to see if we really want to process this event.
+        #
+        processEvent = (orca_state.activeScript == self \
+                        or self.presentIfInactive)
+
+        if not processEvent:
+            return
+
         # This calls the first listener it finds whose key *begins with* or is
         # the same as the event.type.  The reason we do this is that the event
         # type in the listeners dictionary may not be as specific as the event
