@@ -21,12 +21,13 @@
 fallback to local settings if the user settings doesn't exist (e.g., in the
 case of gdm) or doesn't have the specified attribute."""
 
-__id__        = "$Id: settings.py,v 1.80 2006/10/03 08:15:22 wwalker Exp $"
-__version__   = "$Revision: 1.80 $"
-__date__      = "$Date: 2006/10/03 08:15:22 $"
+__id__        = "$Id: settings.py,v 1.84 2006/11/02 16:07:46 richb Exp $"
+__version__   = "$Revision: 1.84 $"
+__date__      = "$Date: 2006/11/02 16:07:46 $"
 __copyright__ = "Copyright (c) 2005-2006 Sun Microsystems Inc."
 __license__   = "LGPL"
 
+import os
 import re
 import sys
 try:
@@ -360,6 +361,11 @@ cacheAccessibles        = True
 #
 learnModeEnabled        = False
 
+# The location of the user's preferences. By default this is ~/.orca.
+# It can be overridden by the Orca -d command line option.
+#
+userPrefsDir = os.path.join(os.environ["HOME"], ".orca")
+
 # If non-zero, we use time.sleep() in various places to attempt to
 # free up the global interpreter lock.  Take a look at the following
 # URLs for more information:
@@ -370,6 +376,10 @@ learnModeEnabled        = False
 #
 gilSleepTime            = 0.00001
 
+# If True, use the gidle __blockPreventor() code in atspi.py.
+#
+useBlockPreventor       = False
+
 # If True, we use the bonobo main loop provided by bonobo to handle
 # all events in atspi.py.  If False, we create our own loop.
 #
@@ -472,6 +482,8 @@ setScriptMapping(re.compile(_('[\S\s]*StarOffice[\s\S]*')), "StarOffice")
 setScriptMapping(re.compile(_('soffice.bin')), "StarOffice")
 setScriptMapping(re.compile(_('[Ee]volution')), "Evolution")
 setScriptMapping(re.compile(_('Deer Park')), "Mozilla")
+setScriptMapping(re.compile(_('yelp')), "Mozilla")
 setScriptMapping(re.compile(_('Bon Echo')), "Mozilla")
 setScriptMapping(re.compile(_('Minefield')), "Mozilla")
 setScriptMapping(re.compile(_('bug-buddy')), "gnome_segv2")
+setScriptMapping(re.compile(_('vte')), "gnome-terminal")
