@@ -23,9 +23,9 @@ class.  For those wishing to override the speech generators, however,
 one can create a new instance and replace/extend the speech generators
 as they see fit."""
 
-__id__        = "$Id: speechgenerator.py,v 1.77 2006/09/14 22:57:28 wwalker Exp $"
-__version__   = "$Revision: 1.77 $"
-__date__      = "$Date: 2006/09/14 22:57:28 $"
+__id__        = "$Id: speechgenerator.py,v 1.83 2006/10/25 00:31:48 wwalker Exp $"
+__version__   = "$Revision: 1.83 $"
+__date__      = "$Date: 2006/10/25 00:31:48 $"
 __copyright__ = "Copyright (c) 2005-2006 Sun Microsystems Inc."
 __license__   = "LGPL"
 
@@ -46,8 +46,13 @@ class SpeechGenerator:
     entry point.  Subclasses can feel free to override/extend the
     speechGenerators instance field as they see fit."""
 
-    def __init__(self):
+    def __init__(self, script):
 
+        # The script that created us.  This allows us to ask the
+        # script for information if we need it.
+        #
+        self._script = script
+        
         # Set up a dictionary that maps role names to functions
         # that generate speech for objects that implement that role.
         #
@@ -127,7 +132,7 @@ class SpeechGenerator:
         self.speechGenerators[rolenames.ROLE_TABLE]               = \
              self._getSpeechForTable
         self.speechGenerators[rolenames.ROLE_TABLE_CELL]          = \
-             self._getSpeechForTableCell
+             self._getSpeechForTableCellRow
         self.speechGenerators[rolenames.ROLE_TABLE_COLUMN_HEADER] = \
              self._getSpeechForTableColumnHeader
         self.speechGenerators[rolenames.ROLE_TABLE_ROW_HEADER]    = \
@@ -258,8 +263,11 @@ class SpeechGenerator:
         utterances = []
 
         if not already_focused:
-            utterances.extend(self._getSpeechForObjectLabel(obj))
-            utterances.extend(self._getSpeechForObjectName(obj))
+            label = self._getSpeechForObjectLabel(obj)
+            utterances.extend(label)
+            name = self._getSpeechForObjectName(obj)
+            if name != label:
+                utterances.extend(name)
             utterances.extend(self._getSpeechForObjectRole(obj))
 
         utterances.extend(self._getSpeechForObjectAvailability(obj))
@@ -283,9 +291,11 @@ class SpeechGenerator:
         """
 
         utterances = []
-        utterances.extend(self._getSpeechForObjectLabel(obj))
-        utterances.extend(self._getSpeechForObjectName(obj))
-        utterances.extend(self._getSpeechForObjectRole(obj))
+        label = self._getSpeechForObjectLabel(obj)
+        utterances.extend(label)
+        name = self._getSpeechForObjectName(obj)
+        if name != label:
+            utterances.extend(name)
 
         # Find all the unrelated labels in the dialog and speak them.
         #
@@ -311,9 +321,11 @@ class SpeechGenerator:
         """
 
         utterances = []
-        utterances.extend(self._getSpeechForObjectLabel(obj))
-        utterances.extend(self._getSpeechForObjectName(obj))
-        utterances.extend(self._getSpeechForObjectRole(obj))
+        label = self._getSpeechForObjectLabel(obj)
+        utterances.extend(label)
+        name = self._getSpeechForObjectName(obj)
+        if name != label:
+            utterances.extend(name)
 
         self._debugGenerator("_getSpeechForAnimation",
                              obj,
@@ -358,25 +370,23 @@ class SpeechGenerator:
 
         utterances = []
         if obj.state.count(atspi.Accessibility.STATE_CHECKED):
-            # If it's not already focused, say it's name
-            #
-            if not already_focused:
-                utterances.extend(self._getSpeechForObjectLabel(obj))
-                utterances.extend(self._getSpeechForObjectName(obj))
-                utterances.extend(self._getSpeechForObjectRole(obj))
-                utterances.append(_("checked"))
-                utterances.extend(self._getSpeechForObjectAvailability(obj))
-            else:
-                utterances.append(_("checked"))
+            checkedState = _("checked")
         else:
-            if not already_focused:
-                utterances.extend(self._getSpeechForObjectLabel(obj))
-                utterances.extend(self._getSpeechForObjectName(obj))
-                utterances.extend(self._getSpeechForObjectRole(obj))
-                utterances.append(_("not checked"))
-                utterances.extend(self._getSpeechForObjectAvailability(obj))
-            else:
-                utterances.append(_("not checked"))
+            checkedState = _("not checked")
+
+        # If it's not already focused, say it's name
+        #
+        if not already_focused:
+            label = self._getSpeechForObjectLabel(obj)
+            utterances.extend(label)
+            name = self._getSpeechForObjectName(obj)
+            if name != label:
+                utterances.extend(name)
+            utterances.extend(self._getSpeechForObjectRole(obj))
+            utterances.append(checkedState)
+            utterances.extend(self._getSpeechForObjectAvailability(obj))
+        else:
+            utterances.append(checkedState)
 
         self._debugGenerator("_getSpeechForCheckBox",
                              obj,
@@ -442,9 +452,14 @@ class SpeechGenerator:
         utterances = []
 
         if not already_focused:
-            utterances.extend(self._getSpeechForObjectLabel(obj))
+            label = self._getSpeechForObjectLabel(obj)
+            utterances.extend(label)
+        else:
+            label = None
 
-        utterances.extend(self._getSpeechForObjectName(obj))
+        name = self._getSpeechForObjectName(obj)
+        if name != label:
+            utterances.extend(name)
 
         if not already_focused:
             utterances.extend(self._getSpeechForObjectRole(obj))
@@ -598,8 +613,11 @@ class SpeechGenerator:
         # availability.]]]
         #
         utterances = []
-        utterances.extend(self._getSpeechForObjectLabel(obj))
-        utterances.extend(self._getSpeechForObjectName(obj))
+        label = self._getSpeechForObjectLabel(obj)
+        utterances.extend(label)
+        name = self._getSpeechForObjectName(obj)
+        if name != label:
+            utterances.extend(name)
 
         if obj.image:
             description = obj.image.imageDescription
@@ -853,8 +871,11 @@ class SpeechGenerator:
         utterances = []
 
         if not already_focused:
-            utterances.extend(self._getSpeechForObjectLabel(obj))
-            utterances.extend(self._getSpeechForObjectName(obj))
+            label = self._getSpeechForObjectLabel(obj)
+            utterances.extend(label)
+            name = self._getSpeechForObjectName(obj)
+            if name != label:
+                utterances.extend(name)
             utterances.extend(self._getSpeechForObjectRole(obj))
 
         utterances.append(percentage)
@@ -898,29 +919,22 @@ class SpeechGenerator:
 
         utterances = []
         if obj.state.count(atspi.Accessibility.STATE_CHECKED):
-            # If it's not already focused, say it's name
-            #
-            if not already_focused:
-                # The label is handled as a context in default.py
-                #
-                #utterances.extend(self._getSpeechForObjectLabel(obj))
-                utterances.extend(self._getSpeechForObjectName(obj))
-                utterances.extend(self._getSpeechForObjectRole(obj))
-                utterances.append(_("selected"))
-                utterances.extend(self._getSpeechForObjectAvailability(obj))
-            else:
-                utterances.append(_("selected"))
+            selectionState = _("selected")
         else:
-            if not already_focused:
-                # The label is handled as a context in default.py
-                #
-                #utterances.extend(self._getSpeechForObjectLabel(obj))
-                utterances.extend(self._getSpeechForObjectName(obj))
-                utterances.extend(self._getSpeechForObjectRole(obj))
-                utterances.append(_("not selected"))
-                utterances.extend(self._getSpeechForObjectAvailability(obj))
-            else:
-                utterances.append(_("not selected"))
+            selectionState = _("not selected")
+
+        # If it's not already focused, say it's name
+        #
+        if not already_focused:
+            # The label is handled as a context in default.py
+            #
+            #utterances.extend(self._getSpeechForObjectLabel(obj))
+            utterances.extend(self._getSpeechForObjectName(obj))
+            utterances.append(selectionState)
+            utterances.extend(self._getSpeechForObjectRole(obj))
+            utterances.extend(self._getSpeechForObjectAvailability(obj))
+        else:
+            utterances.append(selectionState)
 
         self._debugGenerator("_getSpeechForRadioButton",
                              obj,
@@ -1130,7 +1144,7 @@ class SpeechGenerator:
         return utterances
 
     def _getSpeechForTableCell(self, obj, already_focused):
-        """Get the speech for a table cell
+        """Get the speech utterances for a single table cell
 
         Arguments:
         - obj: the table
@@ -1150,7 +1164,7 @@ class SpeechGenerator:
         if action:
             for i in range(0, action.nActions):
                 debug.println(debug.LEVEL_FINEST,
-                    "speechgenerator._getSpeechForTableCell " \
+                    "speechgenerator.__getTableCellUtterances " \
                     + "looking at action %d" % i)
                 if action.getName(i) == "toggle":
                     obj.role = rolenames.ROLE_CHECK_BOX
@@ -1159,7 +1173,39 @@ class SpeechGenerator:
                     obj.role = rolenames.ROLE_TABLE_CELL
                     break
 
-        if (len(utterances) == 0) and (not already_focused):
+        utterances.append(util.getDisplayedText(\
+                          util.getRealActiveDescendant(obj)))
+
+        # [[[TODO: WDW - HACK attempt to determine if this is a node;
+        # if so, describe its state.]]]
+        #
+        if obj.state.count(atspi.Accessibility.STATE_EXPANDABLE):
+            if obj.state.count(atspi.Accessibility.STATE_EXPANDED):
+                utterances.append(_("expanded"))
+            else:
+                utterances.append(_("collapsed"))
+
+        self._debugGenerator("_getSpeechForTableCell",
+                             obj,
+                             already_focused,
+                             utterances)
+
+        return utterances
+
+    def _getSpeechForTableCellRow(self, obj, already_focused):
+        """Get the speech for a table cell row or a single table cell
+        if settings.readTableCellRow is False.
+
+        Arguments:
+        - obj: the table
+        - already_focused: False if object just received focus
+
+        Returns a list of utterances to be spoken for the object.
+        """
+
+        utterances = []
+
+        if (not already_focused):
             if settings.readTableCellRow:
                 parent = obj.parent
                 row = parent.table.getRowAtIndex(obj.index)
@@ -1181,28 +1227,16 @@ class SpeechGenerator:
                     for i in range(0, parent.table.nColumns):
                         accRow = parent.table.getAccessibleAt(row, i)
                         cell = atspi.Accessible.makeAccessible(accRow)
-                        utterances.append(util.getDisplayedText(\
-                            util.getRealActiveDescendant(cell)))
+                        showing = cell.state.count( \
+                                          atspi.Accessibility.STATE_SHOWING)
+                        if showing:
+                            utterances.extend(self._getSpeechForTableCell(cell, 
+                                                           already_focused))
                 else:
-                    utterances.append(util.getDisplayedText(\
-                    util.getRealActiveDescendant(obj)))
+                    utterances.extend(self._getSpeechForTableCell(obj,
+                                                           already_focused))
             else:
-                utterances.append(util.getDisplayedText(\
-                    util.getRealActiveDescendant(obj)))
-
-        # [[[TODO: WDW - HACK attempt to determine if this is a node;
-        # if so, describe its state.]]]
-        #
-        if obj.state.count(atspi.Accessibility.STATE_EXPANDABLE):
-            if obj.state.count(atspi.Accessibility.STATE_EXPANDED):
-                utterances.append(_("expanded"))
-            else:
-                utterances.append(_("collapsed"))
-
-        self._debugGenerator("_getSpeechForTableCell",
-                             obj,
-                             already_focused,
-                             utterances)
+                utterances = self._getSpeechForTableCell(obj, already_focused)
 
         return utterances
 
@@ -1305,25 +1339,23 @@ class SpeechGenerator:
 
         utterances = []
         if obj.state.count(atspi.Accessibility.STATE_CHECKED):
-            # If it's not already focused, say it's name
-            #
-            if not already_focused:
-                utterances.extend(self._getSpeechForObjectLabel(obj))
-                utterances.extend(self._getSpeechForObjectName(obj))
-                utterances.extend(self._getSpeechForObjectRole(obj))
-                utterances.append(_("pressed"))
-                utterances.extend(self._getSpeechForObjectAvailability(obj))
-            else:
-                utterances.append(_("pressed"))
+            checkedState = _("pressed")
         else:
-            if not already_focused:
-                utterances.extend(self._getSpeechForObjectLabel(obj))
-                utterances.extend(self._getSpeechForObjectName(obj))
-                utterances.extend(self._getSpeechForObjectRole(obj))
-                utterances.append(_("not pressed"))
-                utterances.extend(self._getSpeechForObjectAvailability(obj))
-            else:
-                utterances.append(_("not pressed"))
+            checkedState = _("not pressed")
+            
+        # If it's not already focused, say it's name
+        #
+        if not already_focused:
+            label = self._getSpeechForObjectLabel(obj)
+            utterances.extend(label)
+            name = self._getSpeechForObjectName(obj)
+            if name != label:
+                utterances.extend(name)
+            utterances.extend(self._getSpeechForObjectRole(obj))
+            utterances.append(checkedState)
+            utterances.extend(self._getSpeechForObjectAvailability(obj))
+        else:
+            utterances.append(checkedState)
 
         self._debugGenerator("_getSpeechForToggleButton",
                              obj,
@@ -1470,6 +1502,7 @@ class SpeechGenerator:
             if parent == stopAncestor:
                 break
             if (parent.role != rolenames.ROLE_FILLER) \
+                and (parent.role != rolenames.ROLE_SECTION) \
                 and (parent.role != rolenames.ROLE_LAYERED_PANE) \
                 and (parent.role != rolenames.ROLE_SPLIT_PANE) \
                 and (parent.role != rolenames.ROLE_SCROLL_PANE) \
@@ -1514,4 +1547,4 @@ class SpeechGenerator:
 
         utterances.reverse()
 
-        return [" ".join(utterances)]
+        return utterances
