@@ -19,9 +19,9 @@
 
 """Provides various utility functions for Orca."""
 
-__id__        = "$Id: util.py,v 1.45 2006/10/05 16:31:03 wwalker Exp $"
-__version__   = "$Revision: 1.45 $"
-__date__      = "$Date: 2006/10/05 16:31:03 $"
+__id__        = "$Id: util.py,v 1.50 2006/10/22 20:51:44 wwalker Exp $"
+__version__   = "$Revision: 1.50 $"
+__date__      = "$Date: 2006/10/22 20:51:44 $"
 __copyright__ = "Copyright (c) 2005-2006 Sun Microsystems Inc."
 __license__   = "LGPL"
 
@@ -42,7 +42,9 @@ import chnames
 import debug
 import input_event
 import orca_state
+import phonnames
 import punctuation_settings
+import pronunciation_dict
 import rolenames
 import settings
 import speech
@@ -50,6 +52,11 @@ import speechserver
 
 from orca_i18n import _ # for gettext support
 
+# Embedded object character used to indicate that an object is
+# embedded in a string.
+#
+EMBEDDED_OBJECT_CHARACTER = u'\ufffc'
+
 def isSameObject(obj1, obj2):
     if (obj1 == obj2):
         return True
@@ -566,13 +573,14 @@ def textLines(obj):
         if not moreLines:
             done = True
 
-def _addRepeatSegment(segment, line):
+def _addRepeatSegment(segment, line, respectPunctuation=True):
     """Add in the latest line segment, adjusting for repeat characters
     and punctuation.
 
     Arguments:
     - segment: the segment of repeated characters.
     - line: the current built-up line to characters to speak.
+    - respectPunctuation: if False, ignore punctuation level.
 
     Returns: the current built-up line plus the new segment, after
     adjusting for repeat character counts and punctuation.
@@ -586,10 +594,11 @@ def _addRepeatSegment(segment, line):
         isPunctChar = False
     count = len(segment)
     if (count >= settings.repeatCharacterLimit) \
-       and (not segment[0] in string.whitespace) \
-       and (isPunctChar and (style <= level)):
-        repeatChar = chnames.getCharacterName(segment[0])
-        line += _(" %d %s characters ") % (count, repeatChar)
+       and (not segment[0] in string.whitespace):
+        if (not respectPunctuation) \
+           or (isPunctChar and (style <= level)):
+            repeatChar = chnames.getCharacterName(segment[0])
+            line += _(" %d %s characters ") % (count, repeatChar)
     else:
         line += segment
 
@@ -623,16 +632,46 @@ def adjustForRepeats(line):
     newLine = u''
     segment = lastChar = line[0]
 
+    multipleChars = False
     for i in range(1, len(line)):
         if line[i] == lastChar:
             segment += line[i]
         else:
+            multipleChars = True
             newLine = _addRepeatSegment(segment, newLine)
             segment = line[i]
 
         lastChar = line[i]
 
-    newLine = _addRepeatSegment(segment, newLine)
+    newLine = _addRepeatSegment(segment, newLine, multipleChars)
+
+    return newLine.encode("UTF-8")
+
+def adjustForPronunciation(line):
+    """Adjust the line to replace words in the pronunciation dictionary,
+    with what those words actually sound like.
+
+    Arguments:
+    - line: the string to adjust for words in the pronunciation dictionary.
+
+    Returns: a new line adjusted for words found in the pronunciation
+    dictionary.
+    """
+
+    line = line.decode("UTF-8")
+    newLine = segment = u''
+
+    for i in range(0, len(line)):
+        if isWordDelimiter(line[i]):
+            if len(segment) != 0:
+                newLine = newLine + pronunciation_dict.getPronunciation(segment)
+            newLine = newLine + line[i]
+            segment = u''
+        else:
+            segment += line[i]
+
+    if len(segment) != 0:
+        newLine = newLine + pronunciation_dict.getPronunciation(segment)
 
     return newLine.encode("UTF-8")
 
@@ -707,6 +746,32 @@ def getFrame(obj):
 
     return obj
 
+def getTopLevel(obj):
+    """Returns the top-level object (frame, dialog ...) containing this
+    object, or None if this object is not inside a top-level object.
+
+    Arguments:
+    - obj: the Accessible object
+    """
+
+    debug.println(debug.LEVEL_FINEST,
+                  "Finding top-level object for source.name="
+                  + obj.accessibleNameToString())
+
+    while obj \
+          and (obj != obj.parent) \
+          and (obj.parent.role != rolenames.ROLE_APPLICATION):
+        obj = obj.parent
+        debug.println(debug.LEVEL_FINEST, "--> obj.name="
+                      + obj.accessibleNameToString())
+
+    if obj and (obj.parent.role == rolenames.ROLE_APPLICATION):
+        pass
+    else:
+        obj = None
+
+    return obj
+
 def getTextLineAtCaret(obj):
     """Gets the line of text where the caret is.
 
@@ -975,6 +1040,22 @@ def findUnrelatedLabels(root):
 
     return sortedLabels
 
+def phoneticSpellCurrentItem(string):
+    """Phonetically spell the current flat review word or line.
+
+    Arguments:
+    - string: the string to phonetically spell.
+    """
+
+    for (index, character) in enumerate(string.decode("UTF-8")):
+        if character.isupper():
+            voice = settings.voices[settings.UPPERCASE_VOICE]
+            character = character.lower()
+        else:
+            voice =  settings.voices[settings.DEFAULT_VOICE]
+        string = phonnames.getPhoneticName(character)
+        speech.speak(string, voice)
+
 def printAncestry(child):
    """Prints a hierarchical view of a child's ancestry."""
 
